/**
 * 登录用户状态管理
 */
import { defineStore } from 'pinia';
import type { BadgeProps } from 'element-plus';
import { mapTree } from 'ele-admin-plus/es';
import type { MenuItem } from 'ele-admin-plus/es/ele-pro-layout/types';
import type { DictionaryData } from '@/api/system/dictionary-data/model';
import { getPublicKey, loginRequest } from '@/api/login';
import { loginInfo, APPID } from '@/config/setting';
import { pwdEncryption } from '@/utils/jsencrypt';
import { getUserInfo, setUserInfo } from '@/utils/token-util';
import { getTrendsAppMenuListRequest } from '@/api/common/menu';
import { useEmbed } from '@/utils/use-embed';

export interface UserState {
  info: null;
  menus: MenuItem[] | null;
  authorities: (string | undefined)[];
  roles: (string | undefined)[];
  dicts: Record<string, DictionaryData[]>;
}

export const useUserStore = defineStore('user', {
  state: (): UserState => ({
    /** 当前登录用户的信息 */
    info: null,
    /** 当前登录用户的菜单 */
    menus: null,
    /** 当前登录用户的权限 */
    authorities: [],
    /** 当前登录用户的角色 */
    roles: [],
    /** 字典数据缓存 */
    dicts: {}
  }),
  actions: {
    /**
     * 请求登录用户的个人信息/权限/角色/菜单
     */
    async fetchUserInfo() {
      const _Pro = import.meta.env.PROD;
      let userInfo;
      if (_Pro) {
        userInfo = getUserInfo()!.userInfo;
      } else {
        const pwdKeyRes = await getPublicKey();
        const params = loginInfo;
        params.password = (await pwdEncryption(
          pwdKeyRes.data,
          params.password
        )) as any;
        userInfo = (await loginRequest(params)).data;
      }

      setUserInfo(
        JSON.stringify({
          appId: _Pro ? getUserInfo()!.appId : APPID,
          userInfo: userInfo
        })
      );
      const menusold = await this.getMenuData();
      this.updateMenuActiveTree(menusold);
      const menus = this.transformMenuData(menusold, null);
      const homePath = this.getFirstMenuPath(menus);
      this.setMenus(menus);
      return { menus, homePath };
    },

    transformMenuData(menus, upLevelId) {
      const { handleFormatMenus } = useEmbed();
      handleFormatMenus(menus);
      return menus.map((item) => {
        return {
          menuId: item.itemId || '',
          parentId: upLevelId || '',
          title: item.itemName || '',
          path: item.path || '',
          component: item.component || '',
          menuType: item.itemType,
          sortNumber: item.sortIndex || 1,
          authority: item.itemCode ? `sys:${item.itemCode}:list` : '',
          icon: item.icon || '',
          hide: item.hidden === 'Y' ? 1 : 0,
          meta: {
            hide: item.hidden !== 'Y',
            icon: item.icon || '',
            title: item.itemName || '',
            hideSidebar: item.hideSidebarFlag || false,
            hideHeader: item.hideHeaderFlag || false,
            active: item.meta?.active
          },
          deleted: 0,
          tenantId: 4,
          createTime: item.createTime || '',
          updateTime: item.modifyTime || '',
          children: item.children
            ? this.transformMenuData(item.children, item.upLevelId)
            : null,
          checked: item.checkedIden || null
        };
      });
    },
    /**
     * 自底向上更新树形菜单，处理 hidden 为 "N" 节点的 meta.active
     * @param {Array} menuList - 菜单树数组，每个节点包含 itemId、hidden、path、itemName、children 等字段
     */
    updateMenuActiveTree(menuList) {
      function findNearestVisibleParent(node) {
        if (!node || node.hidden === 'Y') return node;
        return findNearestVisibleParent(node.parent);
      }
      function traverse(node, parent = null) {
        node.parent = parent;
        if (node.children && node.children.length > 0) {
          node.children.forEach((child) => traverse(child, node));
        }
        if (node.hidden === 'N') {
          const visibleParent = findNearestVisibleParent(parent);
          if (visibleParent) {
            if (!node.meta) {
              node.meta = {};
            }
            node.meta.active =
              visibleParent.path ||
              visibleParent.itemName ||
              visibleParent.itemId;
          }
        }
      }
      menuList.forEach((item) => traverse(item));
    },

    /**
     * @description 获取菜单列表中第一个有效路径
     * @param menus
     * @returns
     */
    getFirstMenuPath(menus) {
      for (const item of menus) {
        if (item.path && item.menuType === 'page') return item.path; // 返回第一个有效路径
        if (item.children) {
          const childPath = this.getFirstMenuPath(item.children);
          if (childPath) return childPath;
        }
      }
      return '/'; // 默认返回根路径
    },

    /**
     * 更新用户信息
     */
    setInfo(value) {
      this.info = value;
    },
    /**
     * 获取应用菜单数据，并格式化菜单、路由
     * @param appId 应用id
     * @returns 返回promise
     */
    async getMenuData(appId: string = APPID) {
      // 根据appid 获取菜单数据
      const res = await getTrendsAppMenuListRequest({ appId });
      if (res.code !== 200) return Promise.reject('获取菜单数据失败');
      if (!res.data.length)
        return Promise.reject('当前应用无菜单,请先为应用分配菜单');
      else return Promise.resolve(res.data);
    },
    /**
     * 更新菜单数据
     */
    setMenus(menus: MenuItem[] | null) {
      this.menus = menus;
    },
    /**
     * 更新菜单的徽章
     * @param path 菜单地址
     * @param value 徽章值
     * @param type 徽章类型
     */
    setMenuBadge(
      path: string,
      value?: number | string | null,
      type?: BadgeProps['type']
    ) {
      this.menus = mapTree(this.menus, (m) => {
        if (path === m.path) {
          const meta = m.meta || {};
          return {
            ...m,
            meta: {
              ...meta,
              props: {
                ...meta.props,
                badge: value == null ? void 0 : { value, type }
              }
            }
          };
        }
        return m;
      });
    },
    /**
     * 更新字典数据
     */
    setDicts(
      value: DictionaryData[] | Record<string, DictionaryData[]>,
      code?: string | null
    ) {
      if (code == null) {
        this.dicts = value as Record<string, DictionaryData[]>;
        return;
      }
      this.dicts[code] = value as DictionaryData[];
    }
  }
});
